
          <h4>Configuration navigation</h4>
          <div id="navbar-config" bs-scrollspy-list>
            <ul class="nav nav-sidebar">
              <li class="active" bs-scrollspy data-target="#boolean"><a href="#boolean">Booleans</a></li>
              <li bs-scrollspy data-target="#string"><a href="#string">Strings</a></li>
              <li bs-scrollspy data-target="#integer"><a href="#integer">Integers</a></li>
              <li bs-scrollspy data-target="#multi"><a href="#multi">Multi</a></li>
            </ul>
          </div>
